/*
  device.cpp

  Copyright (c) 2021 Kevin Matz (kevin.matz@gmail.com)

  Permission is hereby granted, free of charge, to any person obtaining a copy
  of this software and associated documentation files (the "Software"), to deal
  in the Software without restriction, including without limitation the rights
  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
  copies of the Software, and to permit persons to whom the Software is
  furnished to do so, subject to the following conditions:

  The above copyright notice and this permission notice shall be included in all
  copies or substantial portions of the Software.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
  SOFTWARE.
*/

#include "device.h"

namespace DMX {

/**
 * @brief Device::Device
 */
Device::Device()
  : personality_(0)
  , startAddress_(0)
{
}


/**
 * @brief Device::~Device
 */
Device::~Device()
{
  for ( Personality* p : personalities_ )
    delete p;
}


/**
 * @brief Device::footprint
 * @return
 */
uint16_t Device::footprint() const
{
  if (personality_ == 0 || personality_ > personalities_.size())
    return 0;
  return personalities_.at(personality_ - 1)->footprint();
}

/**
 * @brief Device::personality
 * @return
 */
uint8_t  Device::personality() const
{
  return personality_;
}


/**
 * @brief Device::personalityCount
 * @return
 */
uint8_t  Device::personalityCount() const
{
  return personalities_.size();
}


/**
 * @brief Device::address
 * @return
 */
uint16_t Device::address() const
{
  return startAddress_;
}


/**
 * @brief Device::setPersonality
 * @param mode
 */
void Device::setPersonality(uint8_t mode)
{
  personality_ = mode;
}


/**
 * @brief Device::setAddress
 * @param start
 * @return
 */
bool Device::setAddress(uint16_t start)
{
  if (!footprint())
    return false;

  if (start == 0 || (start + (footprint() - 1)) > 512)
    return false;

  startAddress_ = start;
  return true;
}

} // namespace DMX
