/*
  manager.cpp

  Copyright (c) 2021 Kevin Matz (kevin.matz@gmail.com)

  Permission is hereby granted, free of charge, to any person obtaining a copy
  of this software and associated documentation files (the "Software"), to deal
  in the Software without restriction, including without limitation the rights
  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
  copies of the Software, and to permit persons to whom the Software is
  furnished to do so, subject to the following conditions:

  The above copyright notice and this permission notice shall be included in all
  copies or substantial portions of the Software.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
  SOFTWARE.
*/

#include "manager.h"
#include "rdmnet/rdmnet.h"

namespace RDMnet::LLRP {

/**
 * @brief Manager::Manager
 * @param cid
 */
Manager::Manager(UUID::uuid cid)
  : ACN::RLP::Component(cid)
  , transactionNumber(0)
  , probeInactiveOnly(true)
{
  fctn_ = "OpenLCP LLRP Manager";

  RlpRegisterVector(VECTOR_ROOT_LLRP, std::bind(&Manager::rlpLlrpReceiver,
                                                  this, std::placeholders::_1));
}


/**
 * @brief Broker::rlpBrokerReceiver
 * @param root
 */
void Manager::rlpLlrpReceiver(ACN::PDU::Message<ACN::RLP::Pdu> root)
{
  // data segment will be a block of LLRP Pdu
  root->createDataBlock<LLRP::Pdu>();

  auto block = std::static_pointer_cast<ACN::PDU::Block<LLRP::Pdu>>(root->data());
  for(auto const &frame : *block->pdu)
  {
    auto data = std::static_pointer_cast<llrp_data>(frame->data());
    switch (frame->vector())
    {
      case VECTOR_LLRP_PROBE_REPLY:
        receiveProbeReply(std::static_pointer_cast<LLRP::ProbeReply::Pdu>(data->child));
        break;
    }
  }
}


/**
 * @brief sendProbeRequest
 */
void Manager::sendProbeRequest()
{
  return;
};


/**
 * @brief Manager::receiveProbeReply
 * @param pdu
 */
void Manager::receiveProbeReply(ACN::PDU::Message<LLRP::ProbeReply::Pdu> pdu)
{
  (void)pdu;
};

} // namespace RDMnet::LLRP
