/*
  manager.h

  Copyright (c) 2021 Kevin Matz (kevin.matz@gmail.com)

  Permission is hereby granted, free of charge, to any person obtaining a copy
  of this software and associated documentation files (the "Software"), to deal
  in the Software without restriction, including without limitation the rights
  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
  copies of the Software, and to permit persons to whom the Software is
  furnished to do so, subject to the following conditions:

  The above copyright notice and this permission notice shall be included in all
  copies or substantial portions of the Software.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
  SOFTWARE.
*/
#pragma once

#include "llrp.h"
#include "rlp/component.h"

namespace RDMnet::LLRP {

/**
 * @brief 5.1.1 LLRP Manager
 * LLRP Managers issue LLRP discovery probes and issue configuration change
 * commands. LLRP Managers might be found on lighting consoles or other
 * configuration tool equipment.
 */
class Manager
    : public ACN::RLP::Component
{
public:
  Manager(UUID::uuid = UUID::uuid());

  void rlpLlrpReceiver(ACN::PDU::Message<ACN::RLP::Pdu>);

  virtual void sendProbeRequest();
  virtual void receiveProbeReply(ACN::PDU::Message<LLRP::ProbeReply::Pdu>);

private:
  uint32_t transactionNumber;

  /// > \cite RDMnet 5.4.2.1 Probe Request PDU
  /// > LLRP Managers shall set this bit by default and shall only clear
  /// > it as a result of a user-configured setting in a user-generated
  /// > Probe Request.
  bool probeInactiveOnly;
};

} // namespace RDMnet::LLRP
