/*
  sensor.h

  Copyright (c) 2021 Kevin Matz (kevin.matz@gmail.com)

  Permission is hereby granted, free of charge, to any person obtaining a copy
  of this software and associated documentation files (the "Software"), to deal
  in the Software without restriction, including without limitation the rights
  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
  copies of the Software, and to permit persons to whom the Software is
  furnished to do so, subject to the following conditions:

  The above copyright notice and this permission notice shall be included in all
  copies or substantial portions of the Software.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
  SOFTWARE.
*/
#pragma once

#include <cstdint>
#include <string>

#include "message.h"

namespace RDM {

/**
 * @brief The Sensor class
 */
class Sensor
{
public:
  Sensor();
  virtual ~Sensor() {};

  // RDM required properties
  uint8_t type;         //!< value type
  uint8_t unit;         //!< unit type
  uint8_t unitPrefix;   //!< unit prefix type
  int16_t minLimit;     //!< minimum value
  int16_t maxLimit;     //!< maximum value
  int16_t minNominal;   //!< nominal value minimum
  int16_t maxNominal;   //!< nominal value maximum
  union {
    uint8_t recordedValueSupport;
    struct {
      bool recordedValue       : 1;
      bool minMaxDetectedValue : 1;
      uint8_t reserved         : 6;
    };
  };
  std::string description;  //!< sensor description

  // modifiers
  void setValue(int16_t &val);
  void save();
  void clearMemory();

  // queriers
  int16_t value()   const;
  int16_t saved()   const;
  int16_t minimum() const;
  int16_t maximum() const;

  // PID behaviors
  void actionGetSensorDefinition (uint8_t index, MsgPtr response);
  void actionGetSensorValue      (uint8_t index, MsgPtr response);
  void actionSetSensorValue      (uint8_t index, MsgPtr response);
  void actionSetRecordSensors    (MsgPtr response);

private:
  int16_t min_;
  int16_t max_;
  int16_t memory_;
  int16_t value_;
};

} // namespace RDM
