/*
  pdu.h

  Copyright (c) 2021 Kevin Matz (kevin.matz@gmail.com)

  Permission is hereby granted, free of charge, to any person obtaining a copy
  of this software and associated documentation files (the "Software"), to deal
  in the Software without restriction, including without limitation the rights
  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
  copies of the Software, and to permit persons to whom the Software is
  furnished to do so, subject to the following conditions:

  The above copyright notice and this permission notice shall be included in all
  copies or substantial portions of the Software.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
  SOFTWARE.
*/
#pragma once

#include "acn/pdu.h"
#include "rdm/uid.h"

namespace RDMnet {

/**
 * @brief The RDMnet::Pdu class
 */
class Pdu
    : public ACN::PDU::Pdu
{
public:
  /**
   * @brief Pdu
   * @param vector_size
   */
  Pdu(size_t vector_size)
    : ACN::PDU::Pdu(vector_size)
  {
    flags_._raw = 0xF0; // RDMnet PDU flags always 0xF
  }

  /**
   * @brief RDMnet::Pdu::iStream
   * @param stream
   *
   * RDMnet PDU must have flag value of 0xF0
   */
  virtual void iStream(ACN::PDU::Stream stream) override
  {
    ACN::PDU::Pdu::iStream(stream);
    if (flags_._raw != 0xf0)
        stream_->setstate(std::ios_base::failbit);
  };
};

} // namespace RDMNet


ACN::PDU::pdu_stream& operator>>(ACN::PDU::pdu_stream&, RDM::UID&);
ACN::PDU::pdu_stream& operator<<(ACN::PDU::pdu_stream&, const RDM::UID&);
