/*
  rlp/component.h

  Copyright (c) 2021 Kevin Matz (kevin.matz@gmail.com)

  Permission is hereby granted, free of charge, to any person obtaining a copy
  of this software and associated documentation files (the "Software"), to deal
  in the Software without restriction, including without limitation the rights
  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
  copies of the Software, and to permit persons to whom the Software is
  furnished to do so, subject to the following conditions:

  The above copyright notice and this permission notice shall be included in all
  copies or substantial portions of the Software.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
  SOFTWARE.
*/
#pragma once

#include "acn/component.h"
#include "acn/pdu.h"
#include "rlp.h"
#include "sdt/udp.h"

#include <map>
#include <memory>
#include <vector>


namespace ACN::RLP {

/**
 * @brief The RLP::Component class
 *
 * The process, program or application corresponding to a single ACN endpoint.
 * All messages in ACN are sent and received by a component which is identified
 * by a CID.
 */
class Component
  : public ACN::Component
{
public:
  Component(UUID::uuid = UUID::uuid());

  // EPI 17 - ACN RLP on UDP
  virtual void UdpPayloadReceiver(PDU::Stream);
  void rlpSendUdp(const uint32_t vector, const PDU::Message<PDU::pdu_data> data,
                  const SDT::UDP::ipAddress&);

  // EPI 33 - ACN RLP Operation on TCP
  virtual void TcpPacketReceiver(PDU::Stream);
  void rlpSendTcp(const uint32_t vector, const PDU::Message<PDU::pdu_data> data,
                  const SDT::UDP::ipAddress&);

protected:
  void RlpReceiver(PDU::Message<RLP::Pdu>);
  void RlpRegisterVector(uint32_t, PDU::Handler<RLP::Pdu>);

  void rlpSend(ACN::RLP::message_transport&, const uint32_t vector,
               const PDU::Message<PDU::pdu_data> data,
               const SDT::UDP::ipAddress&);

  virtual void rlpSend(const PDU::Stream, const SDT::UDP::ipAddress&);

private:
  std::map<uint32_t, std::vector<PDU::Handler<RLP::Pdu>>> rlp_vectors_;
};

}; // ACN::RLP
