/*
  rlp.cpp

  Copyright (c) 2020 Kevin Matz (kevin.matz@gmail.com)

  Permission is hereby granted, free of charge, to any person obtaining a copy
  of this software and associated documentation files (the "Software"), to deal
  in the Software without restriction, including without limitation the rights
  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
  copies of the Software, and to permit persons to whom the Software is
  furnished to do so, subject to the following conditions:

  The above copyright notice and this permission notice shall be included in all
  copies or substantial portions of the Software.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
  SOFTWARE.
*/

#include "rlp.h"

namespace ACN::RLP {

/**
 * @brief rlp_header::streamSize
 * @return
 */
size_t rlp_header::streamSize() const
{
    return UUID_LENGTH;
}


/**
 * @brief rlp_header::iStream
 * @param stream
 */
void rlp_header::iStream(PDU::Stream stream)
{
  if (!stream->good())
    return;

  *stream >> cid;
}


/**
 * @brief rlp_header::oStream
 * @param stream
 */
void rlp_header::oStream(PDU::Stream stream) const
{
  *stream << cid;
}


/**
 * @brief RLP::Pdu::Pdu
 * @param stream
 */
Pdu::Pdu()
  : ACN::PDU::Pdu(4)
{
}


/**
 * @brief Pdu::iStream
 * @param stream
 */
void Pdu::iStream(PDU::Stream stream)
{
  PDU::Pdu::iStream(stream);          // flags, length, and vector
  createHeader<rlp_header>();         // header
  // protocol dependent data segment  // data
}


/**
 * @brief message_transport::iStream
 * @param stream
 */
void message_transport::iStream(PDU::Stream stream)
{
  *stream >> root;
}


/**
 * @brief message_transport::oStream
 * @param stream
 */
void message_transport::oStream(PDU::Stream stream) const
{
  *stream << root;
}

} // ACN::RLP
