/*
  rlp/udp.h

  Copyright (c) 2020 Kevin Matz (kevin.matz@gmail.com)

  Permission is hereby granted, free of charge, to any person obtaining a copy
  of this software and associated documentation files (the "Software"), to deal
  in the Software without restriction, including without limitation the rights
  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
  copies of the Software, and to permit persons to whom the Software is
  furnished to do so, subject to the following conditions:

  The above copyright notice and this permission notice shall be included in all
  copies or substantial portions of the Software.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
  SOFTWARE.
*/
#pragma once

#include "rlp.h"
#include "acn/pdu.h"

// ACN EPI 17. ACN Root Layer Protocol Operation on UDP

namespace ACN::RLP::UDP {

/**
 * @brief ACN_PACKET_IDENTIFIER
 * The ACN Packet Identifier shall be the text string “ASC-E1.17\0\0\0”
 * encoded in [ASCII].
 */
static const uint8_t ACN_PACKET_IDENTIFIER[] = { 0x41, 0x53, 0x43, 0x2d,
                                                 0x45, 0x31, 0x2e, 0x31,
                                                 0x37, 0x00, 0x00, 0x00 };
/**
 * @brief PREAMBLE_MINIMUM_SIZE
 * The preamble size includes both size fields so the minimum value for
 * preamble size is 16 (octets).
 */
static const uint8_t PREAMBLE_MINIMUM_SIZE = 16;


/**
 * @brief 2. Preamble Format
 */
struct transport
    : RLP::message_transport
{
  transport(bool filled = false);

  uint16_t length = 0;          //!< length of preamble + PDU block
  uint16_t postamble_size = 0;  //!< length of postamble
  uint8_t  acn_id[12];          //!< packet identifier

  size_t streamSize() const override { return PREAMBLE_MINIMUM_SIZE
                                              + root.streamSize(); }
  void   iStream(PDU::Stream) override;
  void   oStream(PDU::Stream) const override;
};


} // ACN::RLP::UDP
