/*
  sdt/channel.h

  Copyright (c) 2021 Kevin Matz (kevin.matz@gmail.com)

  Permission is hereby granted, free of charge, to any person obtaining a copy
  of this software and associated documentation files (the "Software"), to deal
  in the Software without restriction, including without limitation the rights
  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
  copies of the Software, and to permit persons to whom the Software is
  furnished to do so, subject to the following conditions:

  The above copyright notice and this permission notice shall be included in all
  copies or substantial portions of the Software.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
  SOFTWARE.
*/
#pragma once

#include "acn/component.h"

#include <memory>

namespace ACN::SDT {

/**
 * @brief 3.3 Sequenced Channels
 *  Sequenced channels transport three categories of traffic:
 */
enum Direction {
  internal,   // SDT internal traffic
  downstream, // Session downstream traffic
  upstream    // Session upstream traffic
};


/**
 * @brief The Channel class
 *  Sequenced channels are unidirectional communication channels (unicast or
 *  multicast) from an owner component to one or more member components.
 */
class Channel {
public:
  Channel(std::shared_ptr<Component>, Direction);
  ~Channel();

private:
  std::shared_ptr<Component> owner_;
  Direction direction_ = internal;
};

} // namespace ACN::SDT
