/*
  sdt/leader.h

  Copyright (c) 2021 Kevin Matz (kevin.matz@gmail.com)

  Permission is hereby granted, free of charge, to any person obtaining a copy
  of this software and associated documentation files (the "Software"), to deal
  in the Software without restriction, including without limitation the rights
  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
  copies of the Software, and to permit persons to whom the Software is
  furnished to do so, subject to the following conditions:

  The above copyright notice and this permission notice shall be included in all
  copies or substantial portions of the Software.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
  SOFTWARE.
*/
#pragma once

#include "session.h"
#include "rlp/component.h"

#include <vector>
#include <memory>

namespace ACN::SDT {

/**
 * @brief The Leader class
 */
class Leader
    : public ACN::RLP::Component
{
public:
  Leader(UUID::uuid = UUID::uuid());

  // RLP->SDT frames
  virtual void SdtReceiver(PDU::Message<RLP::Pdu>);
  // SDT 4.4 SDT Base Layer Messages
  // Join, Get Sessions, and Sessions are ad-hoc messages, that is, they are
  // not sent within channels. These messages identify their intended recipient
  // by CID.
  virtual void SdtJoinSession();
  virtual void SdtGetSessions();
  virtual void SdtSessions();

private:
  std::vector<std::shared_ptr<Session>> sdt_sessions_;
};

} // namespace ACN::SDT
