/*
  sdt/udp.cpp

  Copyright (c) 2020 Kevin Matz (kevin.matz@gmail.com)

  Permission is hereby granted, free of charge, to any person obtaining a copy
  of this software and associated documentation files (the "Software"), to deal
  in the Software without restriction, including without limitation the rights
  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
  copies of the Software, and to permit persons to whom the Software is
  furnished to do so, subject to the following conditions:

  The above copyright notice and this permission notice shall be included in all
  copies or substantial portions of the Software.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
  SOFTWARE.
*/

#include "sdt.h"
#include "udp.h"

namespace ACN::SDT::UDP {

/**
 * @brief ipAddress::streamSize
 * @return
 */
size_t ipAddress::streamSize() const
{
  switch (type)
    {
    case SDT_ADDR_IPV4:
      return 7;
    case SDT_ADDR_IPV6:
      return 19;
    case SDT_ADDR_NULL:
    default:
      return 1;
    }
}


/**
 * @brief address_t::iStream
 * @param stream
 */
void ipAddress::iStream(PDU::Stream stream)
{
  *stream >> type;
  if (type == SDT_ADDR_NULL)
    return;

  *stream >> port;
  switch (type)
    {
    case SDT_ADDR_IPV4:
      *stream >> address.ipv4.value;
      break;
    case SDT_ADDR_IPV6:
      for (size_t i = 0;  i < sizeof(address.ipv6.bytes); i++)
        *stream >> address.ipv6.bytes[i];
      break;
  }
}


/**
 * @brief ipAddress::oStream
 * @param stream
 */
void ipAddress::oStream(PDU::Stream stream) const
{
  *stream << type;
  if (type == SDT_ADDR_NULL)
    return;

  *stream << port;
  switch (type)
    {
    case SDT_ADDR_IPV4:
      *stream << address.ipv4.value;
      break;
    case SDT_ADDR_IPV6:
      stream->write(address.ipv6.bytes, sizeof(address.ipv6.bytes));
      break;
  }
}

}; // ACN::SDT::UDP
